/***************************************************************************
 *   Copyright (C) 2018 by Stephane List                                   *
 *   stephane.list@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDebug>
#include <QSettings>
#include <QTextStream>
#include <QMessageBox>
#include <QRegularExpression>
#include <QFileDialog>
#include <QHostAddress>
#include <QByteArray>
#include <QXmlStreamReader>

#include <arpa/inet.h>

#include "version.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    cnt_filter(0), cnt_nat(0), cnt_if(0), cnt_services(0), cnt_services_group(0), cnt_host(0), cnt_net(0), cnt_group(0),
    cnt_eol_val(0), cnt_eol_table(0)
{
    ui->setupUi(this);
    setWindowTitle(windowTitle() + " (V" + VERSION + ")");

    QSettings settings;

    // Disable unsupported Input
    for( int i=0;i<ui->comboBox->count();i++) {
        if ((ui->comboBox->itemText(i) == "Juniper") ||
                (ui->comboBox->itemText(i) == "Fortinet") ||
                (ui->comboBox->itemText(i) == "CheckPoint") ||
                (ui->comboBox->itemText(i) == "PaloAlto") ||
                (ui->comboBox->itemText(i) == "SonicWall") ||
                (ui->comboBox->itemText(i) == "WatchGuard") ||
                (ui->comboBox->itemText(i) == "Zyxel")) {
            ui->comboBox->setItemData(i,false, Qt::UserRole - 1);
        }
    }

    hide_layout(ui->horizontalLayout_iptable);
    hide_layout(ui->horizontalLayout_cisco);

    QFile file_iptable;
    file_iptable.setFileName(settings.value("file_iptable",tr("<Choose file>")).toString());
    if (file_iptable.exists()) {
        ui->lineEdit_iptable->setText(file_iptable.fileName());
    } else {
        ui->lineEdit_iptable->setText(tr("<Choose file>"));
    }

    QFile file_xml1;
    file_xml1.setFileName(settings.value("file_xml1",tr("<Choose file>")).toString());
    if (file_xml1.exists()) {
        ui->lineEdit_xml1->setText(file_xml1.fileName());
    } else {
        ui->lineEdit_xml1->setText(tr("<Choose file>"));
    }

    QFile file_eol;
    file_eol.setFileName(settings.value("file_eol",tr("<Choose file>")).toString());
    if (file_eol.exists()) {
        ui->lineEdit_eol->setText(file_eol.fileName());
    } else {
        ui->lineEdit_eol->setText(tr("<Choose file>"));
    }

    QFile file_cisco;
    file_cisco.setFileName(settings.value("file_cisco",tr("<Choose file>")).toString());
    if (file_cisco.exists()) {
        ui->lineEdit_cisco->setText(file_cisco.fileName());
    } else {
        ui->lineEdit_cisco->setText(tr("<Choose file>"));
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pushButton_iptable_clicked()
{
    QString f = QFileDialog::getOpenFileName(this, tr("Open File"),  QDir::homePath(),
                                             tr("Text (*.txt)"));

    if (f.isNull())
        return;

    ui->lineEdit_iptable->setText(f);

    QSettings settings;
    settings.setValue("file_iptable", f);
}

void MainWindow::on_pushButton_gen_clicked()
{
    reset_all();
    if (ui->comboBox->currentText() == "AMON") {
        extract_from_eol();
        extract_dns();
        extract_ip();
        extract_from_xml1();

        ui->statusBar->showMessage(""
                                   + QString::number(cnt_eol_val) + " eole variables, "
                                   + QString::number(cnt_eol_table) + " eole tables, "
                                   + QString::number(cnt_host) + " hosts, "
                                   + QString::number(cnt_net) + " network, "
                                   + QString::number(cnt_if) + " interfaces, "
                                   + QString::number(cnt_services) + " services, "
                                   + QString::number(cnt_services_group) + " service groups, "
                                   + QString::number(cnt_filter) + " filter rules, "
                                   + QString::number(cnt_nat) + " nat rules");
    } else if (ui->comboBox->currentText() == "iptables") {
        extract_from_iptable();
        ui->statusBar->showMessage(""
                                   + QString::number(cnt_host) + " hosts, "
                                   + QString::number(cnt_net) + " network");
    } else if (ui->comboBox->currentText() == "Cisco") {
        extract_from_cisco();
        ui->statusBar->showMessage(""
                                   + QString::number(cnt_host) + " hosts, "
                                   + QString::number(cnt_net) + " network, "
                                   + QString::number(cnt_group) + " groups");
    }


    line_highlight();     // Comment = line beginning with nop
    text_highlight("XX"); // TODO
    text_highlight("%%"); // to replace with Amon variable
    text_highlight("NOT "); // NOT adding...
    text_highlight("block"); // TODO : remove
    //   text_highlight("nat"); // TODO : remove
    text_highlight("default_policy"); // TODO : remove
  //  text_highlight("comment"); // TODO : remove
    text_highlight("Error"); // TODO : remove

    showMaximized();
}

void MainWindow::on_actionAbout_QT_triggered()
{
    QMessageBox::aboutQt(this, "any2sns");
}

void MainWindow::on_actionAbout_any2sns_triggered()
{
    QMessageBox::about(this,
                       tr("About"),
                       tr("<p><b>any2sns</b></p>"
                          "<p>Version any2sns: %1"
                          "<p>Version %2: %3.%4"
                          "</p>")
                       .arg(VERSION)
                   #ifdef __GNUC__
                       .arg("GCC")
                       .arg(__GNUC__)
                       .arg(__GNUC_MINOR__)
                   #elif defined(_MSC_VER)
                       .arg("VC++")
                       .arg(_MSC_VER / 100)
                       .arg(_MSC_VER % 100,2,10,QChar('0'))
                   #else
                       .arg("Compiler")
                       .arg("?")
                       .arg("?")
                   #endif
                       );
}

void MainWindow::on_action_Quit_triggered()
{
    QApplication::quit();
}

void MainWindow::on_pushButton_clicked()
{
    ui->textEdit->clear();
}

void MainWindow::on_pushButton_xml1_clicked()
{
    QString f = QFileDialog::getOpenFileName(this, tr("Open File"),  QDir::homePath(), "*");

    if (f.isNull())
        return;

    ui->lineEdit_xml1->setText(f);

    QSettings settings;
    settings.setValue("file_xml1", f);
}

void MainWindow::extract_from_eol()
{
    QFile f(ui->lineEdit_eol->text());

    if (!f.exists()) {
        ui->textEdit->append(f.fileName() + " doesn't exist");
        return;
    }

    QByteArray myByteArray;
    if(!f.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qDebug() << "Can not open " << f.fileName();
        return;
    }

    while (!f.atEnd()) {
        myByteArray.append(f.readLine());
    }

    myByteArray.replace("}, ","\n");
    myByteArray.remove(0, 1); // remove first "{"
    myByteArray.remove(myByteArray.size()-2, 2); // remove last "}}"

    //qDebug().noquote() << myByteArray;
    f.close();
    //  ui->textEdit->append(myByteArray);

    ui->textEdit->append("nop -- Extract AMON variable from file " + f.fileName());

    QTextStream ts(&myByteArray);
    QString line;
    QRegularExpression re_name("\"([A-Za-z0-9_\\./\\-\\s]*)\"");

    //   "adresse_ip_eth0": {"owner": "zephir", "val": "194.214.210.19"
    QRegularExpression re_val("\"([A-Za-z0-9_\\./\\-\\s]*)\"$");

    //    "route_adresse": {"owner": "zephir", "val": ["192.168.0.0", "10.0.0.0", "172.16.0.0", "161.48.0.0"]
    QRegularExpression re_array("\"([A-Za-z0-9\\.]*)\"");

    QRegularExpressionMatch match;
    QRegularExpressionMatchIterator matchi;

    while (ts.readLineInto(&line)) {

        // Skip amon version
        if (line.contains("__version__")) {
            int i = line.indexOf("\",");
            line=line.mid(i+2);
        }
        // ui->textEdit->append("\n" + line);

        match = re_name.match(line);
        if (match.hasMatch()) {
            QString s_name = match.captured(1);
            //            ui->textEdit->append("name= " + s_name);

            match = re_val.match(line);
            if (match.hasMatch()) {
                QString sval = match.captured(1);
                //                ui->textEdit->append("val= " + sval);
                map_eole_var.insert(s_name, sval);
                cnt_eol_val++;
            } else {
                bool found_val = false;
                //int i=0;
                QString s,pos;
                QStringList sl;
                matchi = re_array.globalMatch(line);
                while (matchi.hasNext()) {
                    match = matchi.next();
                    if (found_val) {
                        sl << match.captured(1);
                        //                        ui->textEdit->append(s_name + "[" + QString::number(i++) + "]= " + match.captured(1));
                    } else {
                        if (match.captured(1) == "val") {
                            found_val = true;
                        }
                    }
                }
                if (sl.length() > 0) {
                    map_eole_var.insert(s_name, sl.at(0));
                    map_eole_table.insert(s_name, sl);
                    cnt_eol_table++;
#if 0
                    if (s_name == "adresse_ip_dns") {
                        QString hn;
                        for ( auto ip : sl ) {
                            if (ip == "8.8.8.8") {
                                hn = "dns1.google.com";
                            } else if (ip == "8.8.4.4") {
                                hn = "dns2.google.com";
                            } else {
                                hn = ip;
                                hn = "Host_" + hn.replace(".","_");
                                ui->textEdit->append("CONFIG OBJECT HOST NEW name=" + hn + " IP=" + ip + " resolve=static");
                                cnt_host++;
                            }
                            ui->textEdit->append("CONFIG DNS SERVER ADD " + hn );
                        }
                        ui->textEdit->append("CONFIG DNS ACTIVATE");

                    }
#endif
                } else {
                    ui->textEdit->append("nop %% NO VAL for " + s_name);
                }
            }
        }
    }
    ui->textEdit->append("nop -- Extracted " + QString::number(cnt_eol_val) + " values and " + QString::number(cnt_eol_table) + " tables");
}


void MainWindow::extract_dns()
{
    QString hn;
    QStringList sl = map_eole_table.value("adresse_ip_dns");

    ui->textEdit->append("nop ---\nnop --- Add DNS\nnop ---");

    for ( auto ip : sl ) {
        if (ip == "8.8.8.8") {
            hn = "dns1.google.com";
        } else if (ip == "8.8.4.4") {
            hn = "dns2.google.com";
        } else {
            hn = ip;
            hn = "Host_" + hn.replace(".","_");
            ui->textEdit->append("CONFIG OBJECT HOST NEW name=" + hn + " IP=" + ip + " resolve=static");
            cnt_host++;
        }
        ui->textEdit->append("CONFIG DNS SERVER ADD " + hn );
    }
    ui->textEdit->append("CONFIG DNS ACTIVATE");
}

void MainWindow::extract_ip()
{
    QString interface;
    QStringList sl;
    QString ip, mask;
    quint8 cidr;

    ui->textEdit->append("nop ---\nnop --- Add IP to interfaces ---");


    for (int i=0; i<9; i++) {
        interface = "eth" + QString::number(i);
        ui->textEdit->append("nop Looking for adresse_ip_" + interface);

        ip=map_eole_var.value("adresse_ip_" + interface);
        mask=map_eole_var.value("adresse_netmask_" + interface);
        if ((!ip.isEmpty()) && (!mask.isEmpty())) {
            cidr = netmask_to_cidr(mask);
            if (cidr != 0) {
                ui->textEdit->append("CONFIG NETWORK INTERFACE ADDRESS ADD ifname=ethernet" + QString::number(i)
                                     + " address=" + ip
                                     + " mask=" + QString::number(cidr));

                ui->textEdit->append("CONFIG NETWORK INTERFACE UPDATE state=1 ifname=ethernet" + QString::number(i));
            } else {
                ui->textEdit->append("nop error in mask for IP : " + ip);
            }
        }
    }
    ui->textEdit->append("CONFIG NETWORK INTERFACE ACTIVATE");
}


void MainWindow::extract_from_iptable()
{
    QFile f(ui->lineEdit_iptable->text());

    if (!f.exists()) {
        ui->textEdit->append(f.fileName() + " doesn't exist");
        return;
    }

    QTextStream ts(&f);

    f.open(QIODevice::ReadOnly|QIODevice::Text);

    QString line;
    QRegularExpression re_host("([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})$");
    QRegularExpression re_net("([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})\\/([0-9]{1,3})");

    QRegularExpressionMatch match;


    while (ts.readLineInto(&line)) {

        if (line.contains("Generated by iptables-save")) // Escape iptable version ;-)
            continue;

        match = re_host.match(line);
        if (match.hasMatch()) {
            QString s1 = match.captured(1);
            QString s2 = match.captured(2);
            QString s3 = match.captured(3);
            QString s4 = match.captured(4);
            map_host.insert(s1 + "." + s2 +  "." + s3 + "." + s4, "Host_" + s1 + "_" + s2 +  "_" + s3 + "_" + s4);
        }

        match= re_net.match(line);
        if (match.hasMatch()) {
            QString s1 = match.captured(1);
            QString s2 = match.captured(2);
            QString s3 = match.captured(3);
            QString s4 = match.captured(4);
            QString s5 = match.captured(5);

            if (s5 == "32") {
                map_host.insert(s1 + "." + s2 +  "." + s3 + "." + s4, "Host_" + s1 + "_" + s2 +  "_" + s3 + "_" + s4);
            } else {
                map_net.insert(s1 + "." + s2 +  "." + s3 + "." + s4 + "/" + s5, "Net_" + s1 + "_" + s2 +  "_" + s3 + "_" + s4);
            }
        }
    }

    f.close();
    ui->textEdit->append("nop ---\nnop --- Add Hosts\nnop ---");

    {
        QMapIterator<QString, QString> i(map_host);
        while (i.hasNext()) {
            i.next();
            ui->textEdit->append("CONFIG OBJECT HOST NEW name=" + i.value() + " IP=" + i.key() + " resolve=static");
            cnt_host++;
        }
    }

    ui->textEdit->append("nop ---\nnop --- Add Network\nnop ---");

    {
        QMapIterator<QString, QString> i(map_net);
        QString sep = "/";
        while (i.hasNext()) {
            i.next();
            int pos = i.key().indexOf(sep);
            QString net_ip = i.key().left(pos);
            QString net_mask = i.key().mid(pos+1);

            quint32 ipv4Netmask = 0xFFFFFFFF;
            ipv4Netmask <<= 32 - net_mask.toInt();
            QHostAddress qha(ipv4Netmask);
            // qDebug() << net_mask << qha.toString();

            ui->textEdit->append("CONFIG OBJECT NETWORK NEW name=" + i.value() + " IP=" + net_ip + " mask=" + qha.toString());
            cnt_net++;
        }
    }
}


void MainWindow::extract_from_cisco()
{

    qint64 pos;
    bool res;

    QFile f(ui->lineEdit_cisco->text());

    if (!f.exists()) {
        ui->textEdit->append(f.fileName() + " doesn't exist");
        return;
    }

    QTextStream ts(&f);
    f.open(QIODevice::ReadOnly|QIODevice::Text|QIODevice::Unbuffered);
    QString line, group_name;

    // First iteration : find hosts and network

    ui->textEdit->append("nop\nnop Hosts and network\nnop");

    while (ts.readLineInto(&line)) {
        if (line.startsWith("object network ")) {
            QString obj_name, obj_ip, net_mask, obj_description;
            obj_name = line.mid(sizeof("object_network"));

            ts.readLineInto(&line);
            if (line.startsWith(" host")) {
                obj_ip = line.mid(sizeof(" host"));

                // try to find a description
                pos=ts.pos();
                if (pos == -1) ui->textEdit->append("XX Error, pos == -1");
                ts.readLineInto(&line);
                if (line.startsWith(" description")) {
                    obj_description = line.mid(sizeof(" description"));
                    //config object host new name=myhostdetest comment="my comment" ip="5.6.7.8" resolve=static mac=""
                    ui->textEdit->append("CONFIG OBJECT HOST NEW name=" + obj_name + " comment=\"" + obj_description.toLatin1() + "\" IP=" + obj_ip + " resolve=static");
                } else {
                    // no description, rewind
                    res = ts.seek(pos);
                    if (!res) ui->textEdit->append("XX Error in seek");
                    ui->textEdit->append("CONFIG OBJECT HOST NEW name=" + obj_name + " IP=" + obj_ip + " resolve=static");
                }
                map_host.insert(obj_ip, obj_name);
                cnt_host++;
            } else if (line.startsWith(" subnet")) {
                obj_ip = line.mid(sizeof(" subnet"));
                int i = obj_ip.indexOf(QString(" "));
                if (i>7) {
                    net_mask = obj_ip.mid(i);
                    obj_ip = obj_ip.left(i);

                    // try to find a description
                    pos=ts.pos();
                    if (pos == -1) ui->textEdit->append("XX Error, pos == -1");
                    ts.readLineInto(&line);
                    if (line.startsWith(" description")) {
                        obj_description = line.mid(sizeof(" description"));
                        // config object network new name=mynetdetest comment="my comment" ip=1.2.3.4 mask=255.254.0.0
                        ui->textEdit->append("CONFIG OBJECT NETWORK NEW name=" + obj_name + " comment=\"" + obj_description + "\" ip=" + obj_ip + " mask=" + net_mask);
                    } else {
                        // no description, rewind
                        res = ts.seek(pos);
                        if (!res) ui->textEdit->append("XX Error in seek");
                        ui->textEdit->append("CONFIG OBJECT NETWORK NEW name=" + obj_name + " ip=" + obj_ip + " mask=" + net_mask);
                    }
                    map_net.insert(obj_ip, obj_name);
                    cnt_net++;
                } else {
                    ui->textEdit->append("Error : could not find mask of subnet");
                }
            }
        }
        else if (line.startsWith(" network-object")) {
            line = line.mid(sizeof(" network-object"));
            if (line.startsWith("host ")) {
                line = line.mid(sizeof("host"));
            } else if (line.startsWith("object")) {
                line = line.mid(sizeof("object"));
            } else {
                ui->textEdit->append("nop " + line);
            }
        }
    }

    ui->textEdit->append("nop\nnop Groups\nnop");
    res = ts.seek(0);
    if (!res) ui->textEdit->append("XX Error in seek");

    while (ts.readLineInto(&line)) {
//        pos=ts.pos(); //ui->textEdit->append("pos = " + QString::number(pos));

        if (line.startsWith("object-group")) {
            QString group_type, group_descrition("");
            group_type = line.mid(sizeof("object-group"));
            int i = group_type.indexOf(QString(" "));
            group_name = group_type.mid(i+1);
            group_type = group_type.left(i);

            // try to find a description
            pos=ts.pos();
            //ui->textEdit->append("pos = " + QString::number(pos));
            if (pos == -1) ui->textEdit->append("XX Error, pos == -1");

            ts.readLineInto(&line);

            // Call pos twice, or it fails !! (QT 5.9)
            pos=ts.pos();// ui->textEdit->append("pos = " + QString::number(pos));
            pos=ts.pos(); //ui->textEdit->append("pos = " + QString::number(pos));


            if (line.startsWith(" description")) {
                group_descrition = line.mid(sizeof(" description"));
            } else {
                group_descrition = "";
   //TODO            res = ts.seek(pos); // no description, rewind
                res = ts.seek(pos); // no description, rewind
                if (!res) ui->textEdit->append("XX Error in seek");
                if (ts.status() != QTextStream::Ok) ui->textEdit->append("XX Error ts.status = " + QString::number(ts.status()));
            }

            if (group_type == "network") {
                ui->textEdit->append("config object group new name=" + group_name + " comment=\"" + group_descrition +"\"");

                //ui->textEdit->append("config object group new name=" + group_name + " comment=\"" + group_descrition +"\"");
                /* config object group new name=mygroup comment="my description"
               config object group addto group=mygroup node=myhostdetest
               config object group addto group=mygroup node=mynetdetest
               config object activate
            */
                cnt_group++;
            } else {
                ui->textEdit->append("nop other group TODO: " + group_type + " " + group_name + " " + group_descrition);
            }
        } else if (line.startsWith(" network-object")) {
            line = line.mid(sizeof(" network-object"));
            if (line.startsWith("host ")) {
                line = line.mid(sizeof("host"));
                ui->textEdit->append("config object group addto group=" + group_name + " node=" + line);
            } else if (line.startsWith("object")) {
                line = line.mid(sizeof("object"));
                ui->textEdit->append("config object group addto group=" + group_name + " node=" + line);
            } else {
                ui->textEdit->append("nop config object group addto group=" + group_name + " node=" + line);
            }
        }
        if (ts.status() != QTextStream::Ok) ui->textEdit->append("XX Error ts.status = " + QString::number(ts.status()));

    }
    f.close();
}


void MainWindow::extract_from_xml1()
{
    extract_interfaces();
    extract_services();
    extract_rules();
}

void MainWindow::extract_interfaces()
{
    QFile file(ui->lineEdit_xml1->text());
    if(!file.open(QFile::ReadOnly | QFile::Text)){
        ui->textEdit->append(file.fileName() + " doesn't exist");
        return;
    }

    QXmlStreamReader reader(&file);

    if (reader.readNextStartElement()) {
        if (reader.name() == "firewall"){
            ui->textEdit->append("nop ---\nnop --- Add interfaces\nnop ---");

            while(reader.readNextStartElement()){
                if(reader.name() == "zones"){
                    while(reader.readNextStartElement()){
                        if(reader.name() == "zone"){
                            QString if_virtname = reader.attributes().value("name").toString();
                            QString if_realname = reader.attributes().value("interface").toString();

                            if (if_realname.startsWith("%%nom_zone_")) {
                                if_realname = if_realname.mid(sizeof("%%nom_zone_")-1);
                            }

                            IfType it;
                            QString stag, ip, vlan_id, index, netmask;
                            int ind;

                            if (if_realname == "lo") {
                                it = IF_LO;
                            } else if (if_realname.contains(".")) {
                                it = IF_VLAN;
                                vlan_id = reader.attributes().value("ip").toString();
                                vlan_id = vlan_id.mid(2); // remove first %%
                                int i = vlan_id.indexOf("[");
                                index = vlan_id.mid(i+1);
                                index.truncate(index.indexOf("]"));
                                ind = index.toInt();
                                vlan_id.truncate(i);
                                stag = map_eole_table.value(vlan_id).at(ind);
                            } else {
                                it = IF_ETH;
                            }

                            QString VlanPhysical;
                            int i;

                            switch (it) {
                            case IF_LO:
                                break;
                            case IF_ETH:
                                if_realname = if_realname.replace("eth", "ethernet");
                                ui->textEdit->append("CONFIG NETWORK INTERFACE RENAME ifname=" + if_realname + " name=" + if_virtname);
                                cnt_if++;
                                break;
                            case IF_VLAN:
                                i = if_realname.indexOf(".");
                                VlanPhysical = if_realname.left(i);
                                VlanPhysical.replace("eth", "ethernet");

                                ip = reader.attributes().value("ip").toString();
                                ip = ip.mid(ip.indexOf(".")+1);
                                ip = map_eole_table.value(ip).at(ind);

                                netmask = reader.attributes().value("netmask").toString();
                                netmask = netmask.mid(netmask.indexOf(".")+1);
                                netmask = map_eole_table.value(netmask).at(ind);

                                ui->textEdit->append("CONFIG NETWORK INTERFACE CREATE state=1 protected=1 mtu=1500 physical=" + VlanPhysical
                                                     + " name=" + if_virtname
                                                     + " tag=" + stag
                                                     + " keepVlanPriority=1 maxThroughput=0 ifname=" + reader.attributes().value("name").toString()
                                                     + " address=" + ip + " mask=" + netmask);
                                cnt_if++;
                                break;
                            default:
                                qWarning("unknown interface type");
                            }

                            reader.readElementText(); // Goto next
                        }
                        else {
                            reader.skipCurrentElement();
                        }
                    }
                }
            }
        }
    }
    else {
        ui->textEdit->append("nop ---\nnop --- XML malformated (" + QString(__FUNCTION__) + ") ! ---");
        reader.raiseError(QObject::tr("Incorrect file"));
    }
    file.close();
}

void MainWindow::extract_services()
{
    ui->textEdit->append("nop ---\nnop --- Add services and group services\nnop ---");

    QFile file(ui->lineEdit_xml1->text());
    if(!file.open(QFile::ReadOnly | QFile::Text)){
        ui->textEdit->append(file.fileName() + " doesn't exist");
        return;
    }

    QXmlStreamReader reader(&file);

    if (reader.readNextStartElement()) {
        if (reader.name() == "firewall"){
            while(reader.readNextStartElement()){
                if(reader.name() == "services"){
                    while(reader.readNextStartElement()){
                        if(reader.name() == "service"){

                            /* example in xml ! <service name="8500" protocol="tcp" ports="8500" id="11" libelle="service 8500" tcpwrapper=""/>
                             TO CREATE :
                            config object service new name=toto comment="service toto TO REMOVE" port=12345 proto=TCP
                            config object service new name=toto2 comment="service toto2 TO REMOVE with port range" port=12345 toport=12348 proto=TCP
*/

                            QString srv_name = "amon_" + reader.attributes().value("name").toString();
                            QString srv_protocol = reader.attributes().value("protocol").toString().toUpper(); // TCP or UDP
                            QString srv_ports = reader.attributes().value("ports").toString(); // ex : 8500 or 138-139
                            QString srv_label = reader.attributes().value("libelle").toString(); // ex : service scribe sur les clients

                            if (!srv_ports.contains("-")) {
                                if ((srv_ports.contains("%")) || (srv_ports == "0") || ((srv_protocol != "TCP") && (srv_protocol != "UDP"))) {
                                    ui->textEdit->append("nop Following services are not supported : ports contains % or 0, protocol != TCP or UDP");
                                    ui->textEdit->append("nop config object service new name=" + srv_name + " comment=\"" + srv_label +"\" port=" + srv_ports + " proto=" + srv_protocol);
                                } else {
                                    list_service.append(srv_name);
                                    ui->textEdit->append("config object service new name=" + srv_name + " comment=\"" + srv_label +"\" port=" + srv_ports + " proto=" + srv_protocol);
                                    cnt_services++;
                                }
                            } else {
                                int i = srv_ports.indexOf("-");
                                QString srv_port_from = srv_ports.left(i);
                                QString srv_port_to = srv_ports.mid(i+1);
                                if ((srv_ports.contains("%")) || (srv_port_from == "0") || ((srv_protocol != "TCP") && (srv_protocol != "UDP"))) {
                                    ui->textEdit->append("nop Following services are not supported : ports contains % or 0, protocol != TCP or UDP");
                                    ui->textEdit->append("nop config object service new name=" + srv_name + " comment=\"" + srv_label +"\" port=" + srv_port_from + " toport=" + srv_port_to +" proto=" + srv_protocol);
                                } else {
                                    list_service.append(srv_name);
                                    ui->textEdit->append("config object service new name=" + srv_name + " comment=\"" + srv_label +"\" port=" + srv_port_from + " toport=" + srv_port_to +" proto=" + srv_protocol);
                                    cnt_services++;
                                }
                            }
                            QString s = reader.readElementText(); // Goto next service
                        } else if (reader.name() == "groupe") {

                            /* TARGET =
                             *
config object servicegroup new name=grp_port_test comment="mon groupe de port de test"
config object servicegroup addto group=grp_port_test node=ssh
config object servicegroup addto group=grp_port_test node=ssh_udp
config object servicegroup addto group=grp_port_test node=bittorrent_tcp
config object activate
*/

                            QString grp_name = "amon_" + reader.attributes().value("id").toString();
                            QString grp_label = reader.attributes().value("libelle").toString();

                            ui->textEdit->append("nop ---\nnop --- Add group :" + grp_name + "\nnop ----");

                            ui->textEdit->append("config object servicegroup new name=" + grp_name + " comment=\"" + grp_label +"\"");
                            cnt_services_group++;
                            while(reader.readNextStartElement()){
                                if(reader.name() == "service"){
                                    QString srv_name = "amon_" + reader.attributes().value("name").toString();
                                    if (list_service.contains(srv_name)) {
                                        ui->textEdit->append("config object servicegroup addto group=" + grp_name + " node=" + srv_name);
                                    } else {
                                        ui->textEdit->append("nop Unknow service");
                                        ui->textEdit->append("nop config object servicegroup addto group=" + grp_name + " node=" + srv_name);
                                    }
                                }
                                reader.readElementText(); // Goto next service in groupe
                            }
                            reader.readElementText(); // Goto next groupe
                        }
                        else {
                            ui->textEdit->append("nop ignoring " + reader.name().toString());
                            reader.skipCurrentElement();
                        }
                    }
                } else {
                    reader.skipCurrentElement();
                }
            }
        }
        ui->textEdit->append("config object activate");
    }
    else {
        ui->textEdit->append("nop ---\nnop --- XML malformated (" + QString(__FUNCTION__) + ") ! ---");
        reader.raiseError(QObject::tr("Incorrect file"));
    }
    file.close();
}


void MainWindow::extract_rules()
{
    ui->textEdit->append("nop ---\nnop --- Add rules\nnop ---");

    /* example in xml
     *
     <flux-list>
     <flux zoneA="bastion" zoneB="exterieur">
     <montantes default_policy="0">
     <directive tag="ActiverNGINX" service="scribe-posh" priority="1" action="2" attrs="17" src_inv="0" dest_inv="0" serv_inv="0" libelle="ouverture de posh a travers de nginx" ipsec="0" accept="0">
     <source name="exterieur"/>
     <destination name="bastion"/>
     </directive>

*/

    QFile file(ui->lineEdit_xml1->text());
    if(!file.open(QFile::ReadOnly | QFile::Text)){
        qDebug() << "Cannot read file" << file.errorString();
        return;
    }
    int position = 1;
    int position_nat = 1;
    QXmlStreamReader reader(&file);

    if (reader.readNextStartElement()) {
        if (reader.name() == "firewall"){
            while(reader.readNextStartElement()){
                if(reader.name() == "flux-list"){
                    while(reader.readNextStartElement()){
                        if(reader.name() == "flux"){
                            QString za= reader.attributes().value("zoneA").toString();
                            QString zb= reader.attributes().value("zoneB").toString();
                            QString src,dst,pos;

                            ui->textEdit->append("nop ---\nnop Add rule " + za + " -> " + zb);

                            while(reader.readNextStartElement()){
                                if (reader.name() == "montantes"){
                                    src= "Network_" + za;
                                    dst= "Network_" + zb;
                                    QString zd= reader.attributes().value("default_policy").toString();
                                    if (zd =="1") {
                                        ui->textEdit->append("config filter rule insert type=filter state=on ruleName=\"\" action=pass srctarget=" + src
                                                             + " dsttarget=" + dst
                                                             + " index=5"
                                                             + " global=0"
                                                             + " position=" + QString::number(position++)
                                                             + " comment=\"default_policy\"");
                                    }
                                } else if (reader.name() == "descendantes"){
                                    src= "Network_" + zb;
                                    dst= "Network_" + za;
                                    QString zd= reader.attributes().value("default_policy").toString();
                                    if (zd =="1") {
                                        ui->textEdit->append("config filter rule insert type=filter state=on ruleName=\"\" action=pass srctarget=" + src
                                                             + " dsttarget=" + dst
                                                             + " index=5"
                                                             + " global=0"
                                                             + " position=" + QString::number(position++)
                                                             + " comment=\"default_policy\"");
                                    }
                                }  else {
                                    ui->textEdit->append("nop ignoring " + reader.name().toString());
                                    reader.skipCurrentElement();
                                }
                                while(reader.readNextStartElement()){
                                    if (reader.name() == "directive"){
                                        int d_action = reader.attributes().value("action").toInt();
                                        QString action;
                                        QString d_service = reader.attributes().value("service").toString();
                                        QString libelle= reader.attributes().value("libelle").toString();

                                        if (libelle == "pas de description")
                                            libelle.clear();

                                        switch (d_action) {
                                        case 1:
                                            action = "block";
                                            break;
                                        case 2: // pass
                                        case 4: // pass + NAT
                                            action = "pass";
                                            break;
                                        case 16:
                                            action="nat";
                                            break;
                                        default:
                                            action = "block";
                                            ui->textEdit->append("nop %%% Unknow action ! " + QString::number(d_action));
                                            break;
                                        }
                                        if (action == "nat") {
                                            ui->textEdit->append("config filter rule insert type=nat state=on ruleName=" + src.mid(sizeof("Network_")-1) + "_nat"
                                                                 + " action=" + action
                                                                 + " srctarget=" + src + " dsttarget=" + dst
                                                                 + " dstif=exterieur"
                                                                 + " natsrctarget=Firewall_exterieur"
                                                                 + " natsrcport=ephemeral_fw"
                                                                 + " natdsttarget=internet"
                                                                 + " index=5"
                                                                 + " global=0"
                                                                 + " position=" + QString::number(position_nat++)
                                                                 + " comment=\"" + libelle + "\"");
                                        } else {
                                            ui->textEdit->append("config filter rule insert type=filter state=on ruleName=\"\" action=" + action
                                                                 + " srctarget=" + src
                                                                 + " dsttarget=" + dst
                                                                 + " dstport=" + d_service
                                                                 + " securityinspection=1"
                                                                 + " index=5"
                                                                 + " global=0"
                                                                 + " position=" + QString::number(position++)
                                                                 + " comment=\"" + libelle + "\"");
                                        }
                                        reader.skipCurrentElement();
                                    } else {
                                        ui->textEdit->append("nop ignoring " + reader.name().toString());
                                        reader.skipCurrentElement();
                                    }
                                }
                            }
                        } else {
                            reader.skipCurrentElement();
                        }
                    }
                } else {
                    reader.skipCurrentElement();
                }
            }
        }
        else {
            ui->textEdit->append("nop ---\nnop --- XML malformated (" + QString(__FUNCTION__) + ") ! ---");
            reader.raiseError(QObject::tr("Incorrect file"));
        }
    }
    file.close();
    //ui->textEdit->append("nop ---\nnop --- Added " + QString::number(position-1) + " filter rules and " + QString::number(position_nat-1) + " nat rules");
    cnt_filter = position -1;
    cnt_nat = position_nat -1;
}

void MainWindow::text_highlight(const QString &searchString)
{
    QTextDocument *document = ui->textEdit->document();
    QTextCursor highlightCursor(document);
    QTextCursor cursor(document);
    QTextCharFormat plainFormat(highlightCursor.charFormat());
    QTextCharFormat colorFormat = plainFormat;

    cursor.beginEditBlock();
    colorFormat.setForeground(Qt::red);
    colorFormat.setBackground(Qt::yellow);

    while (!highlightCursor.isNull() && !highlightCursor.atEnd()) {
        highlightCursor = document->find(searchString, highlightCursor);

        if (!highlightCursor.isNull()) {
            highlightCursor.movePosition(QTextCursor::WordRight, QTextCursor::KeepAnchor);
            highlightCursor.mergeCharFormat(colorFormat);
        }
    }
    cursor.endEditBlock();
}

void MainWindow::line_highlight()
{
    QString searchString = "nop";
    QTextDocument *document = ui->textEdit->document();
    QTextCursor highlightCursor(document);
    QTextCursor cursor(document);

    cursor.beginEditBlock();

    QTextCharFormat plainFormat(highlightCursor.charFormat());
    QTextCharFormat colorFormat = plainFormat;
    colorFormat.setForeground(Qt::white);
    colorFormat.setBackground(Qt::darkGreen);

    while (!highlightCursor.isNull() && !highlightCursor.atEnd()) {
        highlightCursor = document->find(searchString, highlightCursor,
                                         QTextDocument::FindWholeWords);

        if (!highlightCursor.isNull()) {
            highlightCursor.movePosition(QTextCursor::EndOfLine,
                                         QTextCursor::KeepAnchor);
            highlightCursor.mergeCharFormat(colorFormat);
        }
    }
    cursor.endEditBlock();
}

void MainWindow::on_pushButton_eol_clicked()
{
    QString f = QFileDialog::getOpenFileName(this, tr("Open File"),  QDir::homePath(), "*");

    if (f.isNull())
        return;

    ui->lineEdit_eol->setText(f);

    QSettings settings;
    settings.setValue("file_eol", f);
}

void MainWindow::hide_layout(QLayout *ql)
{
    for (int i = 0; i < ql->count(); ++i)
    {
        QWidget *widget = ql->itemAt(i)->widget();
        if (widget != NULL)
        {
            widget->setVisible(false);
        }
        else
        {
            // You may want to recurse, or perform different actions on layouts.
            // See gridLayout->itemAt(i)->layout()
        }
    }
}

void MainWindow::show_layout(QLayout *ql)
{
    for (int i = 0; i < ql->count(); ++i)
    {
        QWidget *widget = ql->itemAt(i)->widget();
        if (widget != NULL)
        {
            widget->setVisible(true);
        }
        else
        {
            // You may want to recurse, or perform different actions on layouts.
            // See gridLayout->itemAt(i)->layout()
        }
    }
}

void MainWindow::on_comboBox_highlighted(const QString &src)
{
    if (src == "iptables") {
        hide_layout(ui->horizontalLayout_eol);
        hide_layout(ui->horizontalLayout_xml1);
        hide_layout(ui->horizontalLayout_cisco);
        show_layout(ui->horizontalLayout_iptable);
    }
    if (src == "AMON") {
        show_layout(ui->horizontalLayout_eol);
        show_layout(ui->horizontalLayout_xml1);
        hide_layout(ui->horizontalLayout_iptable);
        hide_layout(ui->horizontalLayout_cisco);
    }
    if (src == "Cisco") {
        hide_layout(ui->horizontalLayout_eol);
        hide_layout(ui->horizontalLayout_xml1);
        hide_layout(ui->horizontalLayout_iptable);
        show_layout(ui->horizontalLayout_cisco);
    }
}

void MainWindow::on_pushButton_2_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),
                                                    QDir::home().absolutePath(),
                                                    tr("Script File (*.script)"));
    if (fileName.isNull())
        return;
    QFile ffile(fileName);
    if (ffile.open(QFile::WriteOnly | QIODevice::Text)) {
        QTextStream out(&ffile);
        out << ui->textEdit->toPlainText();
        ui->statusBar->showMessage("Saved in " + fileName);
    }
}


quint8 MainWindow::netmask_to_cidr(const QString &mask)
{
    QStringList list = mask.split('.');

    if (list.count() < 4)
        return 0;

    unsigned short netmask_cidr = 0;
    int ipbytes[4];

    for (int i=0; i<4; i++)
    {
        ipbytes[i] = list[i].toInt();

        switch(ipbytes[i])
        {
        case 0x80:
            netmask_cidr+=1;
            break;

        case 0xC0:
            netmask_cidr+=2;
            break;

        case 0xE0:
            netmask_cidr+=3;
            break;

        case 0xF0:
            netmask_cidr+=4;
            break;

        case 0xF8:
            netmask_cidr+=5;
            break;

        case 0xFC:
            netmask_cidr+=6;
            break;

        case 0xFE:
            netmask_cidr+=7;
            break;

        case 0xFF:
            netmask_cidr+=8;
            break;

        default:
            return netmask_cidr;
            break;
        }
    }
    return netmask_cidr;
}


void MainWindow::on_pushButton_cisco_clicked()
{
    QString f = QFileDialog::getOpenFileName(this, tr("Open File"),  QDir::homePath(),
                                             tr("Text (*.txt)"));

    if (f.isNull())
        return;

    ui->lineEdit_cisco->setText(f);

    QSettings settings;
    settings.setValue("file_cisco", f);
}

void MainWindow::reset_all()
{
    cnt_host = 0;
    cnt_net = 0;
    cnt_group = 0;
    cnt_eol_table = 0;
    cnt_eol_val = 0;
    cnt_filter = 0;
    cnt_if=0;
    cnt_nat = 0;
    cnt_services = 0;
    cnt_services_group = 0;
    map_eole_table.clear();
    map_eole_var.clear();
    map_host.clear();
    map_net.clear();
    ui->textEdit->clear();
}
